@@ -148,8 +148,8 @@ switch (Va->item) {
     CDF->MAXzVars = 0;
     CDF->rVars = NULL;
     CDF->zVars = NULL;
-    CDF->encoding = BOO(DEFAULT_TO_HOST_ENCODING,
-			HostEncoding(),NETWORK_ENCODING);
+    CDF->encoding = BOO(DEFAULT_TO_HOST_ENCODING,HostEncoding(),
+			NETWORK_ENCODING);
     CDF->rowMajor = DEFAULT_TO_ROW_MAJOR;
     CDF->singleFile = DEFAULT_TO_SINGLE_FILE;
     CDF->rMaxRec = NO_RECORD;
@@ -252,6 +252,7 @@ switch (Va->item) {
     if (CDF->checksum > 0) {
         if (!sX(CDFsetChecksum(*id, CDF->checksum),&pStatus)) return pStatus;
     }
+    AddOpenCDFsCount();
     break;
   }
 
