@@ -384,7 +384,6 @@ switch (Va->item) {
     /**************************************************************************
     * Locate the current variable.
     **************************************************************************/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) VDRoffset = CDF->CURzVarOffset;
     else VDRoffset = CDF->CURrVarOffset;
@@ -397,7 +396,6 @@ switch (Va->item) {
     else
         Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
                       CDF->rVars[(int)CDF->CURrVarNum]);
-
     /**************************************************************************
     * Read VDR.
     **************************************************************************/
@@ -505,7 +503,7 @@ switch (Va->item) {
   ****************************************************************************/
   case CDF_CHECKSUM_: {
     long checksumValue = va_arg (Va->ap,long);
-    long version, release;
+    Int32 version, release;
     struct CDFstruct *CDF; Int32 CDRflags;
     SelectCDF (Cur->cdf, CDF)
 
@@ -777,7 +775,8 @@ switch (Va->item) {
 	   CDF->CURgrEntryNum,
 	   CDF->CURgrEntryNum,
 	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
-    if (!ValidDataType(newDataType)) return BAD_DATA_TYPE;
+    if (!ValidDataType(newDataType) || InValidDataType(newDataType))
+      return BAD_DATA_TYPE;
     if (newNumElems != -99 && newNumElems < 1) return BAD_NUM_ELEMS;
     if (!sX(CheckEntryOp(CDF,entryType),&pStatus)) return pStatus;
     eOffset = E3(entryType,CDF->CURgrEntryOffset,
@@ -825,7 +824,8 @@ switch (Va->item) {
 	   CDF->CURgrEntryNum,
 	   CDF->CURgrEntryNum,
 	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
-    if (!ValidDataType((Int32)dataType)) return BAD_DATA_TYPE;
+    if (!ValidDataType((Int32)dataType) || InValidDataType((Int32)dataType))
+      return BAD_DATA_TYPE;
     if (numElems < 1) return BAD_NUM_ELEMS;
     if (!sX(ReadADR(CDF->fp,CDF->CURattrOffset,
 		    ADR_RECORD,&ADR,
