@@ -1,6 +1,6 @@
 /******************************************************************************
 *
-*  NSSDC/CDF                                                    CDFconvert.
+*  NSSDC/CDF                                             CDFconvert.
 *
 *  Version 2.5c, 14-Dec-97, Hughes STX.
 *
@@ -19,14 +19,21 @@
 *   V2.4b 29-Sep-95, J Love     Macintosh dialog filtering.  Outline default
 *                               button.  Less CHECKforABORTso.
 *   V2.5   9-Sep-96, J Love     CDF V2.6.
-*   V2.5a 21-Feb-97, J Love	Removed RICE.
-*   V2.5b 17-Nov-97, J Love	Windows NT/Visual C++.
-*   V2.5c 14-Dec-97, J Love	Added ALPHAVMSi encoding.
+*   V2.5a 21-Feb-97, J Love     Removed RICE.
+*   V2.5b 17-Nov-97, J Love     Windows NT/Visual C++.
+*   V2.5c 14-Dec-97, J Love     Added ALPHAVMSi encoding.
 *   V2.6  17-Apr-01, M Liu      Added checking for variable names entered in
 *                               the compression option
 *   V2.7  03-May-06, M Liu      Added checksum option for the converted files.
 *   V2.8  13-Oct-06, M Liu      Changed to allow upper and lower case CDF  
 *                               name to be used on win32.
+*   V3.3  10-Apr-09, M Liu      Changed MAC_ENCODING to PPC_ENCODING.
+*   V3.3a  7-Jan-10, M Liu      Allowed 'none' compression option to have
+*                               a blocking factor.
+*   V3.32 20-Apr-11, M Liu      Added "epoch2tt2000", "tt20002epoch" or 
+*                               "tt20002epoch16" option to convert epoch from
+*                               CDF_EPOCH/CDF_EPOCH16 to CDF_TIME_TT2000 or 
+*                               from CDF_TIME_TT2000 to CDF_EPOCH/CDF_EPOCH16.
 *
 ******************************************************************************/
 #include "cdfcvt.h"
@@ -41,6 +48,12 @@
 extern unsigned _stklen = BORLANDC_STACK_SIZE;
 #endif
 
+/******************************************************************************
+* Local function prototypes.
+******************************************************************************/
+
+static void ConvertEpoch (long, int, long, double *, long long *);
+
 /******************************************************************************
 * Global variables.
 ******************************************************************************/
@@ -57,6 +70,7 @@ CDFid srcId, dstId, sktId;
 long zMode;
 Logical report[3];
 Logical backward = FALSE;
+long epochvsTT2000 = 0;
 int checksumFlag;
 long checksum, srcChecksum = -999;
 long workingCache, stageCache, compressCache;
@@ -101,11 +115,12 @@ char *argV[];
     "nolog", "percent", "nopercent", "delete", "nodelete", "zmode",
     "neg2posfp0", "noneg2posfp0", "report", "page", "nopage", "cache",
     "statistics", "nostatistics", "encoding", "compression", "sparseness",
-    "about", "backward", "checksum", NULL };
+    "about", "backward", "checksum", "epoch2tt2000", "tt20002epoch",
+    "tt20002depoch", NULL };
   static int optRequired[] = {
     FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE,
     FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE,
-    TRUE, TRUE, TRUE, FALSE, FALSE, TRUE, 0 };
+    TRUE, TRUE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, 0 };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
   CDFstatus status;
   char srcSpec[DU_MAX_PATH_LEN+1];
@@ -218,7 +233,7 @@ char *argV[];
 	  static long encodings[] = {
 	    HOST_ENCODING, NETWORK_ENCODING, SUN_ENCODING, VAX_ENCODING,
 	    DECSTATION_ENCODING, SGi_ENCODING, IBMPC_ENCODING, IBMRS_ENCODING,
-	    MAC_ENCODING, HP_ENCODING, NeXT_ENCODING, ALPHAOSF1_ENCODING,
+	    PPC_ENCODING, HP_ENCODING, NeXT_ENCODING, ALPHAOSF1_ENCODING,
 	    ALPHAVMSd_ENCODING, ALPHAVMSg_ENCODING, ALPHAVMSi_ENCODING
 	  };
 	  int match = FindUniqueMatch (qop->qualOpt[ENCODINGqual],
@@ -319,6 +334,14 @@ char *argV[];
         backward = TRUE;
       }
       /************************************************************************
+      * Check for `epoch2tt2000', `tt20002epoch' or `tt20002epoch16' qualifier.
+      ************************************************************************/
+      qopError = qopError | !S3qualifierLong(qop,&epochvsTT2000,
+                                             EPOCH2TT2000qual,EPOCH2TT2000_,
+                                             TT20002EPOCHqual,TT20002EPOCH_,
+                                             TT20002EPOCH16qual,TT20002EPOCH16_,
+                                             0L,"epoch conversion");
+      /************************************************************************
       * Check for `checksum' qualifier.
       ************************************************************************/
       if (qop->qualEntered[CHECKSUMqual]) {
@@ -375,6 +398,7 @@ char *argV[];
       }
       break;
   }
+
   /****************************************************************************
   * If a skeleton CDF was specified, open it and set the destination format,
   * majority, and encoding (unless they were explicitly specified).
@@ -400,7 +424,7 @@ char *argV[];
        useSkeletonCDF = FALSE;
     }
     if (!PriorTo ("3.2.0", srcVersion, srcRelease, srcIncrement)) {
-      status = CDFlib (CDF_CHECKSUM_, &srcChecksum,
+      status = CDFlib (GET_, CDF_CHECKSUM_, &srcChecksum,
                        NULL_);
       if (StatusBAD(status)) {
          StatusHandlerCvt ("SKT", status);
@@ -529,7 +553,7 @@ char *dstPath;
 			 rVARs_DIMSIZES_, dimSizes,
 		   NULL_);
   if (!StatusHandlerCvt("SRC",status)) return FALSE;
-  if (!PriorTo ("3.2.0", version, release, increment)) {
+  if (!PriorTo ("3.2.0", (Int32) version, (Int32) release, (Int32) increment)) {
     status = CDFlib (GET_, CDF_CHECKSUM_, &srcChecksum,
                      NULL_);
     if (!StatusHandlerCvt("SRC",status)) return FALSE;
@@ -554,10 +578,12 @@ char *dstPath;
           (uSize >= (OFF_T) ((1LL << 31) - 1))) {
 #endif
         WriteOut (stdout, "Soruce file size: ");
-#if !defined(win32)
-        sprintf (text, "cSize(%lld) uSize(%lld)", cSize, uSize);
+#if defined(win32) || defined(__MINGW32__)
+        snprintf (text, (size_t) sizeof(text),
+		  "cSize(%I64d) uSize(%I64d)", cSize, uSize);
 #else
-        sprintf (text, "cSize(%I64d) uSize(%I64d)", cSize, uSize);
+        snprintf (text, (size_t) sizeof(text),
+		  "cSize(%lld) uSize(%lld)", cSize, uSize);
 #endif
         WriteOut (stdout, text);
         WriteOut (stdout, " -- is too big to be converted to a V2.7 file. \n");
@@ -642,11 +668,12 @@ char *dstPath;
       }
     else {					/* not specified */
       int env = CDFgetChecksumEnvVar();
-      if (env == 0) 				/* env var not set */
-        status = CDFlib (SELECT_, CDF_, dstId,
-                         PUT_, CDF_CHECKSUM_, srcChecksum,
-                         NULL_);
-      else					/* env var set to md5 */
+      if (env == 0) {  			/* env var not set ior set to no */
+        if (srcChecksum != -999)
+          status = CDFlib (SELECT_, CDF_, dstId,
+                           PUT_, CDF_CHECKSUM_, srcChecksum,
+                           NULL_);
+      } else					/* env var set to md5 */
         status = CDFlib (SELECT_, CDF_, dstId,
                          PUT_, CDF_CHECKSUM_, MD5_CHECKSUM,
                          NULL_);
@@ -785,7 +812,7 @@ Logical ConvertAttributes ()
 	 WriteOut (stdout, "      Converting entries...\n");
        }
        for (entryNum = 0; entryNum <= maxEntry; entryNum++) {
-	  if (!ConvertEntry(entryNum,gENTRYt)) return FALSE;
+	  if (!ConvertEntry(attrName,entryNum,gENTRYt)) return FALSE;
        }
      }
      else {
@@ -798,10 +825,10 @@ Logical ConvertAttributes ()
 	 WriteOut (stdout, "      Converting entries...\n");
        }
        for (entryNum = 0; entryNum <= maxEntry; entryNum++) {
-	  if (!ConvertEntry(entryNum,rENTRYt)) return FALSE;
+	  if (!ConvertEntry(attrName,entryNum,rENTRYt)) return FALSE;
        }
        for (entryNum = 0; entryNum <= maxZentry; entryNum++) {
-	  if (!ConvertEntry(entryNum,zENTRYt)) return FALSE;
+	  if (!ConvertEntry(attrName,entryNum,zENTRYt)) return FALSE;
        }
      }
      CHECKforABORTso
@@ -814,7 +841,8 @@ Logical ConvertAttributes ()
 * ConvertEntry.
 ******************************************************************************/
 
-Logical ConvertEntry (entryNum, entryType)
+Logical ConvertEntry (attrName, entryNum, entryType)
+char *attrName;
 long entryNum;
 int entryType;
 {
@@ -822,13 +850,25 @@ int entryType;
   long dataType;
   long numElements;
   void *value;
+  int rzEntry;
+  char varName[CDF_VAR_NAME_LEN256+1];
 
+  if (entryType == gENTRYt)
+    rzEntry = 0;
+  else 
+    rzEntry = 1;
   status = CDFlib (SELECT_, CDF_, srcId,
 			    ENTRY(entryType), entryNum,
 		   GET_, ENTRY_DATATYPE(entryType), &dataType,
 			 ENTRY_NUMELEMS(entryType), &numElements,
-		   NULL_);
+                   NULL_);
   if (status != NO_SUCH_ENTRY) {
+    if (rzEntry == 1) {
+      status = CDFlib (SELECT_, (entryType == rENTRYt ? rVAR_ : zVAR_), entryNum,
+                       GET_, (entryType == rENTRYt ? rVAR_NAME_ : zVAR_NAME_),
+                             varName,
+                       NULL_);
+    }
     if (!StatusHandlerCvt("SRC",status)) return FALSE;
     value = cdf_AllocateMemory ((size_t) (CDFelemSize(dataType) * numElements),
 			    FatalError);
@@ -836,12 +876,77 @@ int entryType;
 		     GET_, ENTRY_DATA(entryType), value,
 		     NULL_);
     if (!StatusHandlerCvt("SRC",status)) return FALSE;
-    status = CDFlib (SELECT_, CDF_, dstId,
-			      ENTRY(entryType), entryNum,
-		     PUT_, ENTRY_DATA(entryType), dataType, numElements, value,
-		     NULL_);
-    if (!StatusHandlerCvt("DST",status)) return FALSE;
-    cdf_FreeMemory (value, FatalError);
+    if (epochvsTT2000 == 0 || ((dataType != CDF_EPOCH &&
+                                dataType != CDF_EPOCH16 &&
+                                dataType != CDF_TIME_TT2000) &&
+                               (!StrStrIgCaseX(attrName, "units") &&
+                                !StrStrIgCaseX(attrName, "fillval") &&
+                                !StrStrIgCaseX(attrName, "validmin") &&
+                                !StrStrIgCaseX(attrName, "validmax")))) {
+      status = CDFlib (SELECT_, CDF_, dstId,
+  			        ENTRY(entryType), entryNum,
+		       PUT_, ENTRY_DATA(entryType), dataType, numElements,
+			     value,
+		       NULL_);
+      if (!StatusHandlerCvt("DST",status)) return FALSE;
+    } else {
+      if (epochvsTT2000 == 1 && ((dataType == CDF_EPOCH ||
+                                  dataType == CDF_EPOCH16) ||
+                                 (StrStrIgCaseX(varName, "epoch") &&
+                                  (StrStrIgCaseX(attrName, "units") ||
+                                   StrStrIgCaseX(attrName, "fillval") ||
+                                   StrStrIgCaseX(attrName, "validmin") ||
+                                   StrStrIgCaseX(attrName, "validmax"))))) {
+        if ((dataType == CDF_EPOCH || dataType == CDF_EPOCH16) ||
+            (!StrStrIgCaseX(attrName, "units"))) {
+          long long *valueX;
+          valueX = (long long *) cdf_AllocateMemory ((size_t)numElements * 8, 
+                                                   FatalError);
+          if (valueX == NULL) return FALSE;
+          ConvertEpoch (dataType, (int)numElements, 1L, (double *)value,
+                        valueX);
+          status = CDFlib (SELECT_, CDF_, dstId,
+                                    ENTRY(entryType), entryNum,
+                           PUT_, ENTRY_DATA(entryType), CDF_TIME_TT2000,
+	    	    	         numElements, valueX,
+                           NULL_);
+          cdf_FreeMemory (valueX, FatalError);
+          if (!StatusHandlerCvt("DST",status)) return FALSE;
+        } else {
+          status = CDFlib (SELECT_, CDF_, dstId,
+                                    ENTRY(entryType), entryNum,
+                           PUT_, ENTRY_DATA(entryType), CDF_CHAR,
+                                 2L, "ns",
+                           NULL_);
+          if (!StatusHandlerCvt("DST",status)) return FALSE;
+        }
+      } else if ((epochvsTT2000 == 2 || epochvsTT2000 == 3) &&
+                 dataType == CDF_TIME_TT2000) {
+        double *valueX;
+        int step = (epochvsTT2000 == 2 ? 1 : 2);
+        valueX = (double *) cdf_AllocateMemory ((size_t)numElements * 8 * step,
+                                                FatalError);
+        if (valueX == NULL) return FALSE;
+        ConvertEpoch (dataType, (int)numElements, epochvsTT2000,
+                      valueX, (long long *)value);
+        status = CDFlib (SELECT_, CDF_, dstId,
+                                  ENTRY(entryType), entryNum,
+                         PUT_, ENTRY_DATA(entryType),
+                               (epochvsTT2000 == 2 ? CDF_EPOCH : CDF_EPOCH16),
+                               numElements, valueX,
+                         NULL_);
+        cdf_FreeMemory (valueX, FatalError);
+        if (!StatusHandlerCvt("DST",status)) return FALSE;
+      } else {
+        status = CDFlib (SELECT_, CDF_, dstId,
+                                  ENTRY(entryType), entryNum,
+                         PUT_, ENTRY_DATA(entryType), dataType, numElements,
+                               value,
+                         NULL_);
+        if (!StatusHandlerCvt("DST",status)) return FALSE;
+      }
+      cdf_FreeMemory (value, FatalError);
+    }
   }
   CHECKforABORTso
   return TRUE;
@@ -960,6 +1065,7 @@ long DSTmajority;
    struct GroupStruct groups; struct HyperStruct hyper;
    Logical srcRowMajor = ROWmajor(SRCmajority), switchMajority;
    static char cvtMsg[] = "      converting variable values...    ";
+   long dataTypeX;
    /***************************************************************************
    * Inquire existing variable in source CDF.
    ***************************************************************************/
@@ -984,6 +1090,19 @@ long DSTmajority;
 		      NULL_);
      if (!StatusHandlerCvt("SRC",status)) return FALSE;
    }
+   if (epochvsTT2000 == 0)
+     dataTypeX = dataType;
+   else {
+     if (epochvsTT2000 == 1 && (dataType == CDF_EPOCH ||
+                                dataType == CDF_EPOCH16))
+       dataTypeX = CDF_TIME_TT2000;
+     else if (epochvsTT2000 == 2 && dataType == CDF_TIME_TT2000)
+       dataTypeX = CDF_EPOCH;
+     else if (epochvsTT2000 == 3 && dataType == CDF_TIME_TT2000)
+       dataTypeX = CDF_EPOCH16;
+     else 
+       dataTypeX = dataType;
+   }
    if (Z) {
      status = CDFlib (SELECT_, CDF_, srcId,
 		      GET_, zVAR_NUMDIMS_, &numDims,
@@ -1010,14 +1129,14 @@ long DSTmajority;
    }
    if (Z) {
      status = CDFlib (SELECT_, CDF_, dstId,
-		      CREATE_, zVAR_, varName, dataType, numElements, numDims,
+		      CREATE_, zVAR_, varName, dataTypeX, numElements, numDims,
 				      dimSizes, recVary, dimVarys, &ignoredNum,
 		      NULL_);
      if (!StatusHandlerCvt("DST",status)) return FALSE;
    }
    else {
      status = CDFlib (SELECT_, CDF_, dstId,
-		      CREATE_, rVAR_, varName, dataType, numElements, recVary,
+		      CREATE_, rVAR_, varName, dataTypeX, numElements, recVary,
 				     dimVarys, &ignoredNum,
 		      NULL_);
      if (!StatusHandlerCvt("DST",status)) return FALSE;
@@ -1032,10 +1151,47 @@ long DSTmajority;
 		    NULL_);
    if (status != NO_PADVALUE_SPECIFIED) {
      if (!StatusHandlerCvt("SRC",status)) return FALSE;
-     status = CDFlib (SELECT_, CDF_, dstId,
-		      PUT_, VAR_PADVALUE(Z), padValue,
-		      NULL_);
-     if (!StatusHandlerCvt("DST",status)) return FALSE;
+     if (epochvsTT2000 == 0 || (dataType != CDF_EPOCH &&
+                                dataType != CDF_EPOCH16 &&
+                                dataType != CDF_TIME_TT2000)) {
+       status = CDFlib (SELECT_, CDF_, dstId,
+		        PUT_, VAR_PADVALUE(Z), padValue,
+		        NULL_);
+       if (!StatusHandlerCvt("DST",status)) return FALSE;
+     } else {
+       if (epochvsTT2000 == 1 && (dataType == CDF_EPOCH ||
+                                  dataType == CDF_EPOCH16)) {
+         long long *epoch = cdf_AllocateMemory ((size_t)numElements * 8,
+	  				        FatalError);
+         if (epoch == NULL) return FALSE; 
+         ConvertEpoch (dataType, (int) numElements, 1L,
+                       (double *)padValue, epoch);
+         status = CDFlib (SELECT_, CDF_, dstId,
+                          PUT_, VAR_PADVALUE(Z), epoch,
+                          NULL_);
+         cdf_FreeMemory (epoch, FatalError);
+         if (!StatusHandlerCvt("DST",status)) return FALSE;
+       } else if ((epochvsTT2000 == 2 || epochvsTT2000 == 3) &&
+                  dataType == CDF_TIME_TT2000) {
+         double *epoch;
+         int step = (epochvsTT2000 == 2 ? 1 : 2);
+         epoch = cdf_AllocateMemory ((size_t)numElements * 8 * step,
+                                     FatalError);
+         if (epoch == NULL) return FALSE;
+         ConvertEpoch (dataType, (int) numElements, epochvsTT2000,
+                       epoch, (long long *)padValue);
+         status = CDFlib (SELECT_, CDF_, dstId,
+                          PUT_, VAR_PADVALUE(Z), epoch,
+                          NULL_);
+         cdf_FreeMemory (epoch, FatalError);
+         if (!StatusHandlerCvt("DST",status)) return FALSE;
+       } else {
+         status = CDFlib (SELECT_, CDF_, dstId,
+                          PUT_, VAR_PADVALUE(Z), padValue,
+                          NULL_);
+         if (!StatusHandlerCvt("DST",status)) return FALSE;
+       }
+     }
    }
    cdf_FreeMemory (padValue, FatalError);
    /***************************************************************************
@@ -1180,8 +1336,76 @@ long DSTmajority;
 	       pctOn = TRUE;
 	     }
 	     CHECKforABORTso
-	     status = HYPER_WRITE (dstId, Z, hyper, buffer2);
-	     if (!StatusHandlerCvt("DST",status)) return FALSE;
+             if (epochvsTT2000 == 0 || (dataType != CDF_EPOCH &&
+                                        dataType != CDF_EPOCH16 &&
+                                        dataType != CDF_TIME_TT2000)) {
+	       status = HYPER_WRITE (dstId, Z, hyper, buffer2);
+	       if (!StatusHandlerCvt("DST",status)) return FALSE;
+             } else {
+               if (epochvsTT2000 == 1 && (dataType == CDF_EPOCH ||
+                                          dataType == CDF_EPOCH16)) {
+                 long long *bufferx;
+  	         int ix, values = 1;
+	         for (ix = 0; ix < numDims; ++ix) values *= (int) dimSizes[ix];
+	         bufferx = (long long *) cdf_AllocateMemory ((size_t) hyper.recCount *
+							   values * 8, FatalError);
+	         if (bufferx != NULL) {
+                   ConvertEpoch (dataType, hyper.recCount * values, 1L,
+		                 (double *) buffer2, bufferx);
+                   status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                   if (!StatusHandlerCvt("DST",status)) return FALSE;
+	         } else {
+		   long count;
+		   int step = (dataType == CDF_EPOCH ? 1 : 2);
+		   bufferx = (long long *) cdf_AllocateMemory (values * 8,                                                                     FatalError);
+		   if (bufferx == NULL) return FALSE;
+		   count = hyper.recCount;
+		   hyper.recCount = 1;
+		   for (ix = 0; ix < (int) count; ++ix) {
+		     hyper.recNumber += ix;
+                     ConvertEpoch (dataType, values, 1L,
+		  		   ((double *)buffer2)+step*ix*values, bufferx);
+                     status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                     if (!StatusHandlerCvt("DST",status)) return FALSE;
+		   }
+                 }
+                 cdf_FreeMemory (bufferx, NULL);
+               } else if ((epochvsTT2000 == 2 || epochvsTT2000 == 3) &&
+                          dataType == CDF_TIME_TT2000) {
+                 double *bufferx;
+                 int ix, values = 1;
+                 int step = (epochvsTT2000 == 2 ? 1 : 2);
+                 for (ix = 0; ix < numDims; ++ix) values *= (int) dimSizes[ix];
+                 bufferx = (double *) cdf_AllocateMemory ((size_t) hyper.recCount *
+                                                           values * 8 * step,
+                                                           FatalError);
+                 if (bufferx != NULL) {
+                   ConvertEpoch (dataType, hyper.recCount * values,
+                                 epochvsTT2000, bufferx, (long long *) buffer2);
+                   status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                   cdf_FreeMemory (bufferx, NULL);
+                   if (!StatusHandlerCvt("DST",status)) return FALSE;
+                 } else {                   long count;
+                   bufferx = (double *) cdf_AllocateMemory (values * 8 * step,
+                                                            FatalError);
+                   if (bufferx == NULL) return FALSE;
+                   count = hyper.recCount;
+                   hyper.recCount = 1;
+                   for (ix = 0; ix < (int) count; ++ix) {
+                     hyper.recNumber += ix;
+                     ConvertEpoch (dataType, values, epochvsTT2000,
+                                   bufferx+step*ix*values,
+                                   (long long *) buffer2);
+                     status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                     if (!StatusHandlerCvt("DST",status)) return FALSE;
+                   }
+                 }
+                 cdf_FreeMemory (bufferx, NULL);
+               } else {
+                 status = HYPER_WRITE (dstId, Z, hyper, buffer2);
+                 if (!StatusHandlerCvt("DST",status)) return FALSE;
+               }
+             }
 	     if (pctLog) {
 	       if (!pctOn) WriteOut (stdout, cvtMsg);
 	       WriteOutPct (PCTx(PCT(hyperN,nHypers,3,3),toRec,maxRec));
@@ -1216,11 +1440,39 @@ long DSTmajority;
 	        indices[dimN] = hyper.dimIndices[dimN];
              }
              for (valueN = 0; valueN < nValues; valueN++) {
-	        status = CDFlib (SELECT_, BOO(Z,zVAR_DIMINDICES_,
-					        rVARs_DIMINDICES_), indices,
-			         PUT_, VAR_DATA(Z), value,
-			         NULL_);
-	        if (!StatusHandlerCvt("DST",status)) return FALSE;
+                if (epochvsTT2000 == 0 || (dataType != CDF_EPOCH &&
+                                           dataType != CDF_EPOCH16 &&
+                                           dataType != CDF_TIME_TT2000)) {
+	          status = CDFlib (SELECT_, BOO(Z,zVAR_DIMINDICES_,
+		  			          rVARs_DIMINDICES_), indices,
+			           PUT_, VAR_DATA(Z), value,
+			           NULL_);
+                } else {
+                  if (epochvsTT2000 == 1 && (dataType == CDF_EPOCH ||
+                                             dataType == CDF_EPOCH16)) {
+		    long long epoch;
+                    ConvertEpoch (dataType, 1, 1L, (double *)value, &epoch);
+                    status = CDFlib (SELECT_, BOO(Z,zVAR_DIMINDICES_,
+                                                    rVARs_DIMINDICES_), indices,
+                                     PUT_, VAR_DATA(Z), &epoch,
+                                     NULL_);
+		  } else if ((epochvsTT2000 == 2 || epochvsTT2000 == 3) &&
+                             dataType == CDF_TIME_TT2000) {
+                    double epoch[2];
+                    ConvertEpoch (dataType, 1, epochvsTT2000,
+                                  epoch, (long long *)value);
+                    status = CDFlib (SELECT_, BOO(Z,zVAR_DIMINDICES_,
+                                                    rVARs_DIMINDICES_), indices,
+                                     PUT_, VAR_DATA(Z), epoch,
+                                     NULL_);
+                  } else {
+                    status = CDFlib (SELECT_, BOO(Z,zVAR_DIMINDICES_,
+                                                    rVARs_DIMINDICES_), indices,
+                                     PUT_, VAR_DATA(Z), value,
+                                     NULL_);
+                  }
+                }
+                if (!StatusHandlerCvt("DST",status)) return FALSE;
 	        if (srcRowMajor)
 	          INCRindicesROW (numDims, dimSizes, indices);
 	        else
@@ -1252,8 +1504,76 @@ long DSTmajority;
 	     pctOn = TRUE;
 	   }
 	   CHECKforABORTso
-	   status = HYPER_WRITE (dstId, Z, hyper, buffer1);
-	   if (!StatusHandlerCvt("DST",status)) return FALSE;
+	   if (epochvsTT2000 == 0 || (dataType != CDF_EPOCH &&
+			              dataType != CDF_EPOCH16 &&
+			              dataType != CDF_TIME_TT2000)) {
+	     status = HYPER_WRITE (dstId, Z, hyper, buffer1);
+	     if (!StatusHandlerCvt("DST",status)) return FALSE;
+	   } else {
+             if (epochvsTT2000 == 1 && (dataType == CDF_EPOCH ||
+                                        dataType == CDF_EPOCH16)) {
+  	       long long *bufferx;
+	       int ix, values = 1;
+	       int step = (dataType == CDF_EPOCH ? 1 : 2);
+	       for (ix = 0; ix < numDims; ++ix) values *= dimSizes[ix];
+	       bufferx = (long long *) cdf_AllocateMemory ((size_t)hyper.recCount *
+							   values * 8,
+                                                           FatalError);
+	       if (bufferx != NULL) {
+	         ConvertEpoch (dataType, hyper.recCount * values,
+		  	       1L, (double *)buffer1, bufferx);
+                 status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                 if (!StatusHandlerCvt("DST",status)) return FALSE;
+	       } else {
+	         long count = hyper.recCount;
+	         bufferx = (long long *) cdf_AllocateMemory (values * 8,
+                                                             FatalError);
+	         if (bufferx == NULL) return FALSE;
+	         hyper.recCount = 1;
+	         for (ix = 0; ix < (int) count; ++ix) {
+                   ConvertEpoch (dataType, values, 1L,
+		  	         ((double *) buffer1)+step*values*ix, bufferx);
+		   hyper.recNumber += ix;
+                   status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                   if (!StatusHandlerCvt("DST",status)) return FALSE;
+	         }
+               }
+               cdf_FreeMemory (bufferx, FatalError);
+	     } else if ((epochvsTT2000 == 2 || epochvsTT2000 == 3) &&
+                        dataType == CDF_TIME_TT2000) {
+               double *bufferx;
+               int ix, values = 1;
+               int step = (epochvsTT2000 == 2 ? 1 : 2);
+               for (ix = 0; ix < numDims; ++ix) values *= dimSizes[ix];
+               bufferx = (double *) cdf_AllocateMemory ((size_t)hyper.recCount *
+                                                         values * 8 * step,
+                                                         FatalError);
+               if (bufferx != NULL) {
+                 ConvertEpoch (dataType, hyper.recCount * values,
+                               epochvsTT2000, bufferx, (long long *)buffer1);
+                 status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                 cdf_FreeMemory (bufferx, FatalError);
+                 if (!StatusHandlerCvt("DST",status)) return FALSE;
+               } else {
+                 long count = hyper.recCount;
+                 bufferx = (double *) cdf_AllocateMemory (values * 8 * step,
+                                                          FatalError);
+                 if (bufferx == NULL) return FALSE;
+                 hyper.recCount = 1;
+                 for (ix = 0; ix < (int) count; ++ix) {
+                   ConvertEpoch (dataType, values, epochvsTT2000,
+                                 bufferx, (long long *) buffer1+values*ix);
+                   hyper.recNumber += ix;
+                   status = HYPER_WRITE (dstId, Z, hyper, bufferx);
+                   if (!StatusHandlerCvt("DST",status)) return FALSE;
+                 }
+                 cdf_FreeMemory (bufferx, FatalError);
+               }
+             } else {
+               status = HYPER_WRITE (dstId, Z, hyper, buffer1);
+               if (!StatusHandlerCvt("DST",status)) return FALSE;
+             }
+	   }
 	   if (pctLog) {
 	     if (!pctOn) WriteOut (stdout, cvtMsg);
 	     WriteOutPct (PCTx(PCT(hyperN,nHypers,2,2),toRec,maxRec));
@@ -1318,7 +1638,7 @@ char *options;
      p2 = strchr (p1, ':');
      if (p2 == NULL) { BADs(S) }
      *p2 = NUL;
-     if (strcmpIgCase(p1,"vars") != 0) {
+     if (strcmpIgCase(p1,"vars") == 1) {
        p1 = p2 + 1;
        if (!ParseSparsenessToken(&p1,&p2,&(S->VARs.sRecordsType),&done)) {
 	 BADs(S)
@@ -1327,7 +1647,7 @@ char *options;
        p1 = p2 + 1;
        continue;
      }
-     if (strcmpIgCase(p1,"var") != 0) {
+     if (strcmpIgCase(p1,"var") == 1) {
        nBytes = sizeof(struct sRecordsVarStruct);
        Var = (struct sRecordsVarStruct *) cdf_AllocateMemory (nBytes, FatalError);
        Var->name = NULL;
@@ -1378,15 +1698,15 @@ Logical *done;
     **p2 = NUL;
   }
   sToken = *p1;
-  if (strcmpIgCase(sToken,"sRecords.PAD") != 0) {
+  if (strcmpIgCase(sToken,"sRecords.PAD") == 1) {
     *sRecordsType = PAD_SPARSERECORDS;
     return TRUE;
   }
-  if (strcmpIgCase(sToken,"sRecords.PREV") != 0) {
+  if (strcmpIgCase(sToken,"sRecords.PREV") == 1) {
     *sRecordsType = PREV_SPARSERECORDS;
     return TRUE;
   }
-  if (strcmpIgCase(sToken,"sRecords.NO") != 0) {
+  if (strcmpIgCase(sToken,"sRecords.NO") == 1) {
     *sRecordsType = NO_SPARSERECORDS;
     return TRUE;
   }
@@ -1438,7 +1758,7 @@ char *options;
      p2 = strchr (p1, ':');
      if (p2 == NULL) { BADc(C) }
      *p2 = NUL;
-     if (strcmpIgCase(p1,"cdf") != 0) {
+     if (strcmpIgCase(p1,"cdf") == 1) {
        p1 = p2 + 1;
        if (!ParseCompressionTokenAndBF(&p1,&p2,
 				       &(C->CDF.cType),
@@ -1448,7 +1768,7 @@ char *options;
        p1 = p2 + 1;
        continue;
      }
-     if (strcmpIgCase(p1,"vars") != 0) {
+     if (strcmpIgCase(p1,"vars") == 1) {
        p1 = p2 + 1;
        if (!ParseCompressionTokenAndBF(&p1,&p2,
 				       &(C->VARs.cType),
@@ -1460,7 +1780,7 @@ char *options;
        p1 = p2 + 1;
        continue;
      }
-     if (strcmpIgCase(p1,"var") != 0) {
+     if (strcmpIgCase(p1,"var") == 1) {
        nBytes = sizeof(struct cVarStruct);
        Var = (struct cVarStruct *) cdf_AllocateMemory (nBytes, FatalError);
        Var->name = NULL;
@@ -1525,28 +1845,33 @@ Logical *done;
     count = sscanf (p+1, "%ld:%ld", &v1, &v2);
     if (count < 1) return FALSE;
   }
-  if (strcmpIgCase(cToken,"none") != 0) {
+  if (strcmpIgCase(cToken,"none") == 1) {
+/*
     if (count > 0) return FALSE;
     *cType = NO_COMPRESSION;
     ASSIGNnotNULL (bf, 0)
     ASSIGNnotNULL (reserve, 0)
+*/
+    *cType = NO_COMPRESSION;
+    ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
+    ASSIGNnotNULL (reserve, 0)
     return TRUE;
   }
-  if (strcmpIgCase(cToken,"rle.0") != 0) {
+  if (strcmpIgCase(cToken,"rle.0") == 1) {
     *cType = RLE_COMPRESSION;
     cParms[0] = RLE_OF_ZEROs;
     ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
     ASSIGNnotNULL (reserve, BOO(count > 1,v2,0))
     return TRUE;
   }
-  if (strcmpIgCase(cToken,"huff.0") != 0) {
+  if (strcmpIgCase(cToken,"huff.0") == 1) {
     *cType = HUFF_COMPRESSION;
     cParms[0] = OPTIMAL_ENCODING_TREES;
     ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
     ASSIGNnotNULL (reserve, BOO(count > 1,v2,0))
     return TRUE;
   }
-  if (strcmpIgCase(cToken,"ahuff.0") != 0) {
+  if (strcmpIgCase(cToken,"ahuff.0") == 1) {
     *cType = AHUFF_COMPRESSION;
     cParms[0] = OPTIMAL_ENCODING_TREES;
     ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
@@ -1555,8 +1880,8 @@ Logical *done;
   }
   for (level = 1; level <= 9; level++) {
      char gzipToken[MAX_GZIP_TOKEN_LEN+1];
-     sprintf (gzipToken, "gzip.%d", level);
-     if (strcmpIgCase(cToken,gzipToken) != 0) {
+     snprintf (gzipToken, (size_t) sizeof(gzipToken), "gzip.%d", level);
+     if (strcmpIgCase(cToken,gzipToken) == 1) {
        *cType = GZIP_COMPRESSION;
        cParms[0] = level;
        ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
@@ -1565,7 +1890,7 @@ Logical *done;
      }
   }
 /*
-  if (strcmpIgCase(cToken,"zlib.0") != 0) {
+  if (strcmpIgCase(cToken,"zlib.0") == 1) {
     *cType = ZLIB_COMPRESSION;
     cParms[0] = ZLIB_OF_ZEROs;
     ASSIGNnotNULL (bf, BOO(count > 0,v1,0))
@@ -1610,6 +1935,41 @@ struct CompressionStruct *compression;
   cdf_FreeMemory (compression, FatalError);
 }
 
+/******************************************************************************
+* ConvertEpoch.
+******************************************************************************/
+
+void ConvertEpoch (dataType, numValues, conversionType, value1, value2)
+long dataType;
+int numValues;
+long conversionType;
+double *value1;
+long long *value2;
+{
+  int ix;
+  CDFstatus status = CDF_OK;
+  if (conversionType == 1) { /* CDF_EPOCH/CDF_EPOCH16 to TT2000 */
+    if (dataType == CDF_EPOCH) {
+      for (ix = 0; ix < numValues; ++ix) {
+        *(value2+ix) = CDF_TT2000_from_UTC_EPOCH (*(value1+ix));
+      }
+    } else {
+      for (ix = 0; ix < numValues; ++ix) {
+        *(value2+ix) = CDF_TT2000_from_UTC_EPOCH16 (value1+ix*2);
+      }
+    }
+  } else if (conversionType == 2) { /* TT2000 to CDF_EPOCH */
+    for (ix = 0; ix < numValues; ++ix) {
+      *(value1+ix) = CDF_TT2000_to_UTC_EPOCH (*(value2+ix));
+    }
+  } else if (conversionType == 3) { /* TT2000 to CDF_EPOCH16 */
+    double tmp;
+    for (ix = 0; ix < numValues; ++ix) {
+      tmp = CDF_TT2000_to_UTC_EPOCH16 (*(value2+ix), value1+2*ix);
+    }
+  }
+}
+
 /******************************************************************************
 * StatusHandlerCvt.
 ******************************************************************************/
@@ -1629,7 +1989,8 @@ CDFstatus status;
       CDFlib (SELECT_, CDF_STATUS_, status,
 	      GET_, STATUS_TEXT_, text,
 	      NULL_);
-      sprintf (oText, "ERROR,%s> %s", which, text);
+      snprintf (oText, (size_t) sizeof(oText), "ERROR,%s> %s", which,
+	        text);
       OutputWithMargin (stdout, oText, MAX_SCREENLINE_LEN, 0);
     }
     return FALSE;
@@ -1643,7 +2004,8 @@ CDFstatus status;
       CDFlib (SELECT_, CDF_STATUS_, status,
 	      GET_, STATUS_TEXT_, text,
 	      NULL_);
-      sprintf (oText, "WARNING,%s> %s", which, text);
+      snprintf (oText, (size_t) sizeof(oText), "WARNING,%s> %s", which, 
+	        text);
       OutputWithMargin (stdout, oText, MAX_SCREENLINE_LEN, 0);
     }
     return TRUE;
@@ -1657,7 +2019,8 @@ CDFstatus status;
       CDFlib (SELECT_, CDF_STATUS_, status,
 	      GET_, STATUS_TEXT_, text,
 	      NULL_);
-      sprintf (oText, "INFO,%s> %s", which, text);
+      snprintf (oText, (size_t) sizeof(oText), "INFO,%s> %s", which, 
+	        text);
       OutputWithMargin (stdout, oText, MAX_SCREENLINE_LEN, 0);
     }
     return TRUE;
@@ -1865,7 +2228,7 @@ char **argV[];
 		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
 	
 		MAKEstrARGv (argV, n, "-zmode")
-		sprintf (tempS1, "%d", zModeSrc);
+		snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zModeSrc);
 		MAKEstrARGv (argV, n, tempS1)
 
 		MAKEstrARGv (argV, n, "-report")
