@@ -25,6 +25,7 @@
 *   V1.6  03-May-06, M Liu      Added checksum option for the files.
 *   V1.7  13-Oct-06, M Liu      Changed to allow upper and lower case CDF  
 *                               name to be used on win32.
+*   V3.3  10-Jan-09, M Liu      Validate a file before it is used.
 *
 ******************************************************************************/
 
@@ -173,7 +174,8 @@ char *argV[];
 	   case '2': zMode = zMODEon2; break;
 	   default: {
 	     char tempS[MAX_MESSAGE_TEXT_LEN+1];
-	     sprintf (tempS, "Illegal zMode (%s).", qop->qualOpt[ZMODEqual]);
+	     snprintf (tempS, (size_t) sizeof(tempS),
+		       "Illegal zMode (%s).", qop->qualOpt[ZMODEqual]);
 	     DisplayError (tempS);
 	     qopError = TRUE;
 	   }
@@ -238,6 +240,7 @@ char *argV[];
    /***************************************************************************
    * Edit CDF(s).
    ***************************************************************************/
+   CDFsetValidate (VALIDATEFILEon); 
    if (promptForSpec)
      status = SpecificationPrompt (CDFspec, useFormat, negToPosFp0, zMode,
 				   workingCache, stageCache, compressCache);
@@ -431,38 +434,50 @@ long compressCache;
 	   ReportStatus (status, FALSE);
 	   if (StatusOK(status)) {
 	     char label[DU_MAX_NAME_LEN+13+1];
-	     sprintf (label, " Info for `%s' ", CDFs[IW.itemN]);
-	     sprintf (infoLines[0], "Compression:  %s",
-		      CompressionToken(cType,cParms));
+	     snprintf (label, (size_t) sizeof(label),
+		       " Info for `%s' ", CDFs[IW.itemN]);
+	     snprintf (infoLines[0], (size_t) sizeof(BLANKs78),
+		       "Compression:  %s", CompressionToken(cType,cParms));
 	     if (cType == NO_COMPRESSION) {
 	       if (isLFS)
-#if !defined(win32)
-	         sprintf (infoLines[1], "Uncompressed: %lld bytes", uFileSize2);
+#if !defined(win32) && !defined(__MINGW32__)
+	         snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %lld bytes", uFileSize2);
 #else
-                 sprintf (infoLines[1], "Uncompressed: %I64d bytes", uFileSize2);
+                 snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %I64d bytes", uFileSize2);
 #endif
 	       else
-		 sprintf (infoLines[1], "Uncompressed: %ld bytes", uFileSize1);
+		 snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %ld bytes", uFileSize1);
 	       MakeNUL (infoLines[2]);
 	     }
 	     else {
 	       if (isLFS) {
-#if !defined(win32)
-                 sprintf (EofS(infoLines[0]), " (%ld%%)",
-                          (long) (((OFF_T)100 * cFileSize2) / uFileSize2));
-	         sprintf (infoLines[1], "Compressed:   %lld bytes", cFileSize2);
-	         sprintf (infoLines[2], "Uncompressed: %lld bytes", uFileSize2);
+                 snprintf (EofS(infoLines[0]), 
+			   (size_t) sizeof(BLANKs78)-strlen(infoLines[0]),
+			   " (%ld%%)",
+                           (long) (((OFF_T)100 * cFileSize2) / uFileSize2));
+#if !defined(win32) && !defined(__MINGW32__)
+	         snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Compressed:   %lld bytes", cFileSize2);
+	         snprintf (infoLines[2], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %lld bytes", uFileSize2);
 #else
-                 sprintf (EofS(infoLines[0]), " (%ld%%)",
-                          (long) (((OFF_T)100 * cFileSize2) / uFileSize2));
-                 sprintf (infoLines[1], "Compressed:   %I64d bytes", cFileSize2);
-                 sprintf (infoLines[2], "Uncompressed: %I64d bytes", uFileSize2);
+                 snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Compressed:   %I64d bytes", cFileSize2);
+                 snprintf (infoLines[2], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %I64d bytes", uFileSize2);
 #endif
 	       } else {
-		 sprintf (EofS(infoLines[0]), " (%ld%%)",
-			 ((100L * cFileSize1) / uFileSize1));
-		 sprintf (infoLines[1], "Compressed:   %ld bytes", cFileSize1);
-		 sprintf (infoLines[2], "Uncompressed: %ld bytes", uFileSize1);
+		 snprintf (EofS(infoLines[0]), 
+			   (size_t) sizeof(BLANKs78)-strlen(infoLines[0]),
+			   " (%ld%%)",
+			   ((100L * cFileSize1) / uFileSize1));
+		 snprintf (infoLines[1], (size_t) sizeof(BLANKs78),
+			   "Compressed:   %ld bytes", cFileSize1);
+		 snprintf (infoLines[2], (size_t) sizeof(BLANKs78),
+			   "Uncompressed: %ld bytes", uFileSize1);
 	       }
 	     }
 	     MessageWindow (infoLines, label, LogicalTRUE);
@@ -501,8 +516,8 @@ long compressCache;
 	 else {
 	   char delim, question[DU_MAX_NAME_LEN+15+1];
 	   delim = PickDelimiter (CDFs[IW.itemN],strlen(CDFs[IW.itemN]));
-	   sprintf (question, "Delete CDF %c%s%c ?", delim, CDFs[IW.itemN],
-		    delim);
+	   snprintf (question, (size_t) sizeof(question),
+		     "Delete CDF %c%s%c ?", delim, CDFs[IW.itemN], delim);
 	   if (ConfirmWindow(3,78,question,NULL,FALSE,DELETECDFhelpID)) {
 	     CDFid id; CDFstatus status; int lastItemN;
 	     Logical isLFS;
@@ -555,9 +570,11 @@ char ***CDFs;
 struct ItemWindowStruct *IW;
 {
   int CDFn;
-  sprintf (IW->label, " Specification \"%s\" ", CDFspec);
+  snprintf (IW->label, (size_t) 16+DU_MAX_PATH_LEN+2+1,
+	    " Specification \"%s\" ", CDFspec);
   *nCDFs = CDFdirList (CDFspec, dirS, CDFs);
-  sprintf (IW->hLines[0], "%d CDF%s", *nCDFs, (*nCDFs == 1 ? "" : "s"));
+  snprintf (IW->hLines[0], (size_t) sizeof(BLANKs78),
+	    "%d CDF%s", *nCDFs, (*nCDFs <= 1 ? "" : "s"));
   if (*nCDFs > 0) {
     AllocIW (IW, *nCDFs, *nCDFs, CDFs_NAME_LEN, FatalError);
     for (CDFn = 0; CDFn < *nCDFs; CDFn++) {
@@ -628,11 +645,13 @@ long compressCache;
 	int helpID;
 	delim = PickDelimiter(CDFfullName, strlen(CDFfullName));
 	if (CDFgetFileBackwardEnvVar() == 1) {
-	  sprintf (question, "CDF %c%s%c does not exist.  Create?", delim,
-	           CDFfullName, delim);
+	  snprintf (question, (size_t) sizeof(question),
+	            "CDF %c%s%c does not exist.  Create?", delim,
+	            CDFfullName, delim);
 	} else {
-	  sprintf (question, "CDF %c%s%c does not exist.  Create(V3 or V2.7)?", delim,
-		   CDFfullName, delim);
+	  snprintf (question, (size_t) sizeof(question),
+	            "CDF %c%s%c does not exist.  Create(V3 or V2.7)?", delim,
+		    CDFfullName, delim);
 	  helpID = CREATECDFV23helpID;
 	}
 	if (ConfirmWindow2(3,78,question,NULL,TRUE,helpID)) {
@@ -715,7 +734,8 @@ struct ItemWindowStruct *IWcdfs;
     strcpyX (dirT, (NULstring(CDFdir) ? CURRENTDIRECTORY : CDFdir),
 	     DU_MAX_DIR_LEN);
     delim = PickDelimiter (dirT, strlen(dirT));
-    sprintf (PW.label, " Creating in %c%s%c ", delim, dirT, delim);
+    snprintf (PW.label, (size_t) 13+1+DU_MAX_DIR_LEN+1+1+1,
+	      " Creating in %c%s%c ", delim, dirT, delim);
     PW.value = CDFname;
     PromptWindow (NEWpw, &PW, 0, LogicalTRUE);
     for (;;) {
@@ -742,7 +762,8 @@ struct ItemWindowStruct *IWcdfs;
   AppendToDir (CDFpathT, CDFname);
   strcpyX (PWdim.value, "0:[]", MAX_DIMENSIONALITY_LEN);
   delim = PickDelimiter (CDFname, strlen(CDFname));
-  sprintf (labelDim, " CDF %c%s%c ", delim, CDFname, delim);
+  snprintf (labelDim, (size_t) sizeof(labelDim),
+	    " CDF %c%s%c ", delim, CDFname, delim);
   PromptWindow (NEWpw, &PWdim, (int) strlen(PWdim.value), LogicalTRUE);
   end_pasteboard_update ();	/* It is OK to call this in the case where
 				   the `begin_pasteboard_update' above was
@@ -1068,7 +1089,8 @@ Logical *closed;
 		 if (!browseOnly) {
 			char delim, question[DU_MAX_NAME_LEN+15+1];
 			delim = PickDelimiter (CDFname, strlen(CDFname));
-	   		sprintf (question, "Delete CDF %c%s%c ?", delim, CDFname, delim);
+	   		snprintf (question, (size_t) sizeof(question),
+				  "Delete CDF %c%s%c ?", delim, CDFname, delim);
 	   		if (ConfirmWindow(12,80,question,NULL,FALSE,DELETECDFhelpID)) {
 	     	  status = CDFlib (DELETE_, CDF_,
 			  			       NULL_);
@@ -1195,7 +1217,8 @@ struct ItemWindowStruct *IW;
   /****************************************************************************
   * Encode label with name of CDF.
   ****************************************************************************/
-  sprintf (IW->label, " CDF \"%s\" ", CDFname);
+  snprintf (IW->label, (size_t) 6+DU_MAX_NAME_LEN+2+1,
+	    " CDF \"%s\" ", CDFname);
   /****************************************************************************
   * Inquire CDF.
   ****************************************************************************/
@@ -1217,39 +1240,50 @@ struct ItemWindowStruct *IW;
   /****************************************************************************
   * Encode first line of header.
   ****************************************************************************/
-  sprintf (IW->hLines[0], "Format:   %s", FormatToken(format));
+  snprintf (IW->hLines[0], (size_t) sizeof(BLANKs78),
+	    "Format:   %s", FormatToken(format));
   pad = MAIN_MENU_1st_THIRD_LEN - strlen(IW->hLines[0]);
   if (pad > 0) CatNcharacters (IW->hLines[0], pad, (int) ' ');
-  sprintf (EofS(IW->hLines[0]), "Attributes: %ld (%ldg/%ldv)",
-	   nAttrs, NgAttrs, NvAttrs);
+  snprintf (EofS(IW->hLines[0]),
+	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[0]),
+	    "Attributes: %ld (%ldg/%ldv)", nAttrs, NgAttrs, NvAttrs);
   pad = MAIN_MENU_2nd_THIRD_LEN - strlen(IW->hLines[0]);
   if (pad > 0) CatNcharacters (IW->hLines[0], pad, (int) ' ');
-  sprintf (EofS(IW->hLines[0]), "Compression: %s",
-	   CompressionToken(cType,cParms));
+  snprintf (EofS(IW->hLines[0]),
+	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[0]),
+	    "Compression: %s", CompressionToken(cType,cParms));
 #if 0
   if (cType != NO_COMPRESSION && cPct > 0) {
-    sprintf (EofS(IW->hLines[0]), " (%ld%%)", cPct);
+    snprintf (EofS(IW->hLines[0]),
+	      (size_t) sizeof(BLANKs78)-strlen(IW->hLines[0]),
+	      " (%ld%%)", cPct);
   }
 #endif
   /****************************************************************************
   * Encode second line of header.
   ****************************************************************************/
-  sprintf (IW->hLines[1], "Majority: %s", MajorityToken(majority));
+  snprintf (IW->hLines[1], (size_t) sizeof(BLANKs78),
+	    "Majority: %s", MajorityToken(majority));
   pad = MAIN_MENU_1st_THIRD_LEN - strlen(IW->hLines[1]);
   if (pad > 0) CatNcharacters (IW->hLines[1], pad, (int) ' ');
-  sprintf (EofS(IW->hLines[1]), "Variables:  %ld (%ldr/%ldz)",
-	   NrVars + NzVars, NrVars, NzVars);
+  snprintf (EofS(IW->hLines[1]),
+	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[1]),
+	    "Variables:  %ld (%ldr/%ldz)", NrVars + NzVars, NrVars, NzVars);
   pad = MAIN_MENU_2nd_THIRD_LEN - strlen(IW->hLines[1]);
   if (pad > 0) CatNcharacters (IW->hLines[1], pad, (int) ' ');
-  sprintf (EofS(IW->hLines[1]), "Encoding:    %s", EncodingToken(encoding));
+  snprintf (EofS(IW->hLines[1]),
+	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[1]),
+	    "Encoding:    %s", EncodingToken(encoding));
   /****************************************************************************
   * Encode third line of header.
   ****************************************************************************/
-  sprintf (IW->hLines[2], "Version:  %ld.%ld.%ld",
-	   version, release, increment);
+  snprintf (IW->hLines[2], (size_t) sizeof(BLANKs78),
+	    "Version:  %ld.%ld.%ld", version, release, increment);
   pad = MAIN_MENU_1st_THIRD_LEN - strlen(IW->hLines[2]);
   if (pad > 0) CatNcharacters (IW->hLines[2], pad, (int) ' ');
-  sprintf (EofS(IW->hLines[2]), "Checksum:   %s", ChecksumToken(checksum));
+  snprintf (EofS(IW->hLines[2]),
+	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[2]),
+	    "Checksum:   %s", ChecksumToken(checksum));
   return TRUE;
 }
 
@@ -1268,14 +1302,13 @@ char CDFspec[DU_MAX_PATH_LEN+1];
 #if defined(vms)
     "Examples: RAIN2, RAIN*, [-.SAMPLES], USER5:[CDF1]IJ01A"
 #endif
-#if (defined(unix) && !defined(__CYGWIN__) && !defined(__MINGW32__)) || \
-     defined(posixSHELL)
+#if defined(unix) || defined(posixSHELL)
     "Examples: rain2, rain*, ../samples, ~/CDFs/ij01a"
 #endif
 #if defined(dos)
     "Examples: RAIN2, RAIN*, ..\\SAMPLES, B:\\CDF1\\IJ01A"
 #endif
-#if defined(win32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(win32)
     "Examples: rain2, rain*, ..\\samples, b:\\cdf1\\ij01a"
 #endif
 #if defined(mac)
@@ -1459,7 +1492,7 @@ char **argV[];
 		MAKEbooARGv (argV, n, gWith, "-gwithentries", "-nogwithentries")
 		MAKEbooARGv (argV, n, vWith, "-vwithentries", "-novwithentries")
 		MAKEstrARGv (argV, n, "-zmode")
-		sprintf (tempS1, "%d", zMode);
+		snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
 		MAKEstrARGv (argV, n, tempS1)
 		MAKEstrARGv (argV, n, "-report")
 		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
